'''
This module contains tools for estimating numerical integrals.
'''

import numpy as np
import matplotlib.pyplot as plt

def visualizeRectangles(f, xmin, xmax, N):
    """

    Plot "N" rectangles between "xmin" and "xmax",
    to visualize what happens when we calculate
    a rectangular integral of the function "f".

    Arguments
        f    : a function to be integrated
        xmin : lower limit of integration
        xmax : upper limit of integration
        N    : number of rectangles to use

    Returns
        (nothing), but it makes a plot

    Example:
        def integrand(x)
            return x**2
        plotRectangles(integrand, 0, 2, 5)
    """

    # first, make a high-resolution plot of the function
    xsmooth = np.linspace(xmin, xmax, 500)
    plt.plot(xsmooth, f(xsmooth), linewidth=10, alpha=0.3, color='orchid')

    # next, make a grid spanning xmin to xmax, with N+1 elements
    x = np.linspace(xmin, xmax, N+1)

    # calculate the offsets from x[:-1] to x[1:]
    dx = np.diff(x)

    # loop over rectangles
    for i in range(N):

        # where are the left and right edges of the rectangle?
        left = x[i]
        right = x[i] + dx[i]

        # set the rectangle plotting style
        plotkw = dict(color='black', linewidth=2)

        # plot vertical line from 0 up to curve (left edge of rectangle)
        plt.plot([left, left], [0, f(x[i])], **plotkw)

        # plot another vertical line (right edge of rectangle)
        plt.plot([right, right], [0, f(x[i])], **plotkw)

        # plot horizontal line (top of rectangle)
        plt.plot([left, right], [f(x[i]), f(x[i])], **plotkw)

    # set the x and y labels
    plt.xlabel('x')
    plt.ylabel('f(x)')

    # make sure the plot displays to the screen
    plt.show()

def integrateRectangles(f, xmin, xmax, N):
    """

    Approximate the area under the function "f",
    from "xmin" to "xmax", using N rectangles.

    Arguments
        f    : a function to be integrated
        xmin : lower limit of integration
        xmax : upper limit of integration
        N    : number of rectangles to use

    Returns
        the numerical integral of "f"

    Example:
        def integrand(x)
            return x**2
        integrateRectangles(integrand, 0, 1, 5)
    """

    # These x values represent edges of rectangles.
    # We need (N+1) edges for N rectangles.
    x = np.linspace(xmin, xmax, N + 1)

    # The widths of the rectangles:
    dx = np.diff(x)  # equivalently x[1:] - x[:-1]

    # The heights of the rectangles (using the left of each pair of points)
    h = f(x[:-1])

    # The areas of each rectangle.
    A = dx * h

    # Add up all those areas.
    return np.sum(A)



'''
This module contains tools to download astronomical spectra from SDSS.

Functions:

    getSDSS: Donwload, extract, and return a Sloan Digital Sky Survey (SDSS) spectrum from the web

    getLabel: Returns a metadata label (string) corresponding to a given spectrum obtained from getSDSS

    test: A function to test the connection to the SDSS Server, and either download a
        spectrum or generate a fake spectrum

    fakeSDSS: Generate and return a fake SDSS spectrum (a random-temperature blackbody spectrum)

    planck:  Calculate and return the Planck function for a given temperature

Constants:
    All constants given in SI units

    h: Planck's constant

    k: Boltzmann's Constant

    c: Speed of Light
'''

import matplotlib.pyplot as plt
import numpy as np
import astropy.io.fits as pf


def downloadSDSS(online=True, verbose=True):
    '''
    Return a random spectrum from the Sloan Digital Sky Survey (SDSS).

        Arguments:

            online:  Boolean keyword to enable testing offline *if* the
                spectrum spec-4055-55359-0596.fits is already downloaded
            verbose: Boolean keyword:  if False, omit print statements

        Returns:

            one random SDSS spectrum. The format of this spectrum
            is as a dictionary that contains many key-value pairs.
            Important keys inside this spectrum are highlighted in
            the example below.

        Example:

            # call the function to get a random spectrum
            s = downloadSDSS()

            # the wavelengths are in units of angstroms (=10**-10 m)
            w_angstroms = s['wavelengths']
            w_nm = np.array(w_angstroms)/10.0

            # the fluxes are in units of "1e-17 erg/cm**2/s/angstrom"
            f = s['flux']

    '''

    # choose spectrum to download
    plate = '4055'
    mjd = '55359'
    num = np.random.randint(0,1000)  # choose among the 1000 spectra here
    numstr = str(num)
    # pad number with leading zeros, for a total of 4 digits
    for i in range(len(numstr),4):
        numstr = '0' + numstr

    # construct a URL to be sent to the SDSS server (just doing eboss for now)
    url = 'https://dr14.sdss.org/sas/dr14/eboss/spectro/redux/v5_10_0/spectra/lite/'
    if online:
        url += plate + '/spec-' + plate + '-' + mjd + '-' + numstr + '.fits'

        # update the user what's going on
        if verbose:
            print("Trying to download SDSS spectrum from:\n {}\n".format(url))

        # send the URL to SDSS, and get back a response
        hdulist = pf.open(url)

    else:
        filein = 'spec-4055-55359-0596.fits'
        hdulist = pf.open(filein)

    # temporarily pull out wavelengths, fluxes, and the spectrum id
    logwav = hdulist[1].data['loglam']
    flux = hdulist[1].data['flux']
    ivar = hdulist[1].data['ivar']
    id = hdulist[2].data['SPECOBJID']

    # build spectrum dictionary
    s = {}

    # wavelength in angstroms
    s['wavelengths'] = np.array(10**logwav)

    # flux in 10e-17 erg cm^-2 s^-1 Angstrom^-1
    #  fluxconversion = 1.0e-3/1.0e-17/1e10
    s['flux'] = flux  # *fluxconversion

    # populate the metadata for this fake spectrum
    s['spectrumID'] = str(id[0])
    s['spectro_class'] = hdulist[2].data['class'][0]
    s['spectro_subclass'] = hdulist[2].data['subclass'][0]
    s['z'] = hdulist[2].data['z'][0]

    # which elements of the w and f arrays are in the optical?
    w = 10**logwav
    optical = (w > 3900) & (w < 8000)

    # is optical flux, on average brighter than 1.0e-17 erg/cm**2/s/angstrom?
    if np.mean(flux[optical]) > 1.0:
        # ...if so, return this spectrum
        if verbose:
            print("Returning a spectrum for {}.".format(id))
        return s
    else:
        # ...if not, call getSDSS() to try to get a brighter one
        if verbose:
            print("Yikes, {} was pretty faint. Trying again...".format(id))
        return getSDSS(online=online, verbose=verbose)


def getLabel(spectrum):
    '''
    Create a short string summarizing the basic properties of
    a SDSS spectrum. The spectrum must be in the dictionary
    format created by the SDSS web API.

        Arguments:

            spectrum = a dictionary containing an SDSS spectrum

        Returns:

            a string describing how the SDSS project has
            classified this object

        Example:
            # get a random spectrum
            s = getSDSS()

            # get its label, returned as a string
            label = getLabel(s)
            print label

    '''

    # create an empty label
    label = ""

    # add the identifier for this spectrum
    label += spectrum["spectrumID"] + "\n"

    # what is the basic class of this object?
    label += spectrum['spectro_class']

    # if there are more details, add them in
    if len(spectrum['spectro_subclass']) > 0:
        label += "/" + spectrum['spectro_subclass']
    label += ", "

    # show the redshift (z = v/c) of the object
    label += "redshift={:.2g}".format(spectrum["z"])

    # return that label as a string
    return label


h = 6.626068e-34    # Planck's constant, m**2 kg / s
k = 1.3806503e-23   # Boltzmann's constant, m**2 kg / s**2 / K
c = 299792458.0     # speed of light, m / s

def planck(w_nm, T=5800):
    """
    Compute the thermal emission spectrum.

    Arguments:
     w_nm = wavelength in nm, float or numpy array
     T = temperature in K (set to 5800K by default)

    Returns:
     intensity at the given wavelength(s), in W / m**3 / sr

    Example:
     >> import thermal
     >> print thermal.planck(550, T=5800)
     2.63106560099e+13
    """

    # convert nm to m
    w = w_nm/1.0e9

    # return Planck function
    num = 2 * h *c**2
    den = w**5 * (np.exp(h * c / (w * k * T)) - 1.0)
    return num/den

def fakeSDSS(T='random'):
    try:
        temperature = float(T)
    except (TypeError, ValueError):
        temperature = np.round(np.random.uniform(2000,9000))


    w = np.linspace(300, 1000, 1000)
    f = planck(w, T=temperature)

    # populate a fake SDSS spectrum
    s = {}

    # wavelength in angstroms
    s['wavelengths'] = np.array(w*10)

    # convert W/m^3 to
    fluxconversion = 1.0e-3/1.0e-17/1e10
    s['flux'] = f*fluxconversion

    # populate the metadata for this fake spectrum
    s['spectrumID'] = 'an imaginary SDSS'
    s['spectro_class'] = '{}K emission spectrum'.format(temperature)
    s['spectro_subclass'] = ''
    s['z'] = 0.0

    label = getLabel(s)
    return w, f, label

def testSDSS(**kwargs):
    '''This function tests the getSDSS() function
        and uses the fakeSDSS() function if that fails.'''

    # get a spectrum
    try:
        w, f, l = getSDSS(**kwargs)
        print("Hooray! It seems like the getSDSS() function works!")
    except:
        print("Uh-oh! It seems like something is broken between the SDSS server and you.")
        print(" Please use sdss.fakeSDSS() instead of sdss.getSDSS() to make spectra.")
        w, f, l = fakeSDSS()

    # plot a spectrum
    wmin = 390
    wmax = 800
    mask = (w>wmin) & (w<wmax)
    plt.plot(w[mask],f[mask])
    plt.title(l)
    plt.xlabel("Wavelength (nm)")
    plt.ylabel("Flux ($10^{-17}$ erg cm$^{-2}$ s$^{-1}$ nm$^{-1}$)")
    plt.xlim(wmin, wmax)
    #plt.ylim(0, None)
    plt.show()

def getSDSS(**kwargs):
    '''
    Return a random spectrum from the Sloan Digital Sky Survey (SDSS).

    Returns
    -------
    wavelength : numpy array
        The wavelengths of the spectrum, in nm.
    flux : numpy array
        The fluxes of the spectrum, in 1e-17 erg/cm**2/s/nm
    description : string
        A string description explaining what the spectrum is.
    '''

    spectrum = downloadSDSS(**kwargs)
    label = getLabel(spectrum)

    wavelength_nm = spectrum['wavelengths']/10.0
    flux = spectrum['flux']*10

    return wavelength_nm, flux, label



import numpy as np

def simulate_topography(numCraters = 10, # number of craters
                        widthMax   = 1.0,     # maximal width of Gaussian crater
                        heightMin  = -1.0,    # maximal depth of craters / valleys
                        heightMax  = 2.0):     # maximal height of hills / mountains

    # 1-D Gaussian
    np.random.seed()   # for reproducability
    def gaussian(x, A, mu, sigma):
        return A * np.exp(-(x - mu)**2 / 2.0 / sigma**2)

    # 1-D Gaussian (same thing using lambda)
    #gaussian = lambda x, A, mu, sigma: A * np.exp(-(x - mu)**2 / 2. / sigma**2)

    # Create an array of linearly spaced x values
    xArr = np.linspace(0, 10, 500)  # km

    # Create an array of initially flat landscape (aka filled with 0's)
    yArr = np.zeros_like(xArr)

    # Add craters / mountains to landscape
    for _ in range(numCraters):

        # Amplitude between heightMin and heightMax
        A = np.random.rand() * (heightMax - heightMin) + heightMin

        # Center location of the crater
        center = np.random.rand() * xArr.max()

        # Width of the crater
        sigma = np.random.rand() * widthMax

        # Add crater to landscape!
        yArr += gaussian(xArr, A=A, mu=center, sigma=sigma)

    return xArr, yArr



import pandas as pd
import pkg_resources
path = pkg_resources.resource_filename(__name__, 'data/earth-standard-atmosphere.txt')
table = pd.read_csv(path, sep='\s+')

altitude = table['altitude'].values
temperature = table['temperature'].values



'''
This setup.py file sets up our package to be installable on any computer,
so that folks can `import henrietta` from within any directory.

Thanks to this file, you can...

...tell python to look for `henrietta` in the current directory (which you
can continue to edit), by typing *one* of the following commands:

`pip install -e .`
or
`python setup.py develop`

...move a copy of this code to your site-packages directory, where python will
be able to find it (but you won't be able to keep editing it), by typing *one*
of the following commands:

`pip install .`
or
`python setup.py install`

...upload the entire package to the Python Package Index, so that other folks
will be able to install your package via the simple `pip install henrietta`, by
running the following command:

`python setup.py release`


The template for this setup.py came was pieced together with help from
barentsen, christinahedges, timothydmorton, and dfm. Check them out on github
for more neat tricks!

[`python-packaging`](https://python-packaging.readthedocs.io/en/latest/index.html)
is a pretty useful resource too!
'''

# import our basic setup ingredients
from setuptools import setup, find_packages
import os,sys

# running `python setup.py release` from the command line will post to PyPI
if "release" in sys.argv[-1]:
    os.system("python setup.py sdist")
    # uncomment the next line to test out on test.pypi.com/project/tess-zap
    #os.system("twine upload --repository-url https://test.pypi.org/legacy/ dist/*")
    os.system("twine upload dist/*")
    os.system("rm -rf dist/cu-astr2600*")
    sys.exit()

# a little kludge to get the version number from __version__
exec(open('astr2600/version.py').read())

# run the setup function
setup(
    # people can type `import henrietta` to access this package
    name = "cu-astr2600",
    # this package will only be installed if the current version doesn't exist
    version = __version__,
    # what's a short description of the package?
    description = "A collection of toys and tools for ASTR2600 at CU Boulder.",
    # what's a more detailed description?
    long_description = open('README.md').read(),
    # who's the main author?
    author = "Zach Berta-Thompson",
    # what's the main author's email?
    author_email = "zach.bertathompson@colorado.edu",
    # what's the URL for the repository?
    url = "https://github.com/zkbt/cu-astr2600",
    # this figures out what subdirectories to include
    packages = find_packages(),
    # are the directories of data that should be accessible when installed?
    include_package_data=True,
    # where are those data directories?
    package_data = {'astr2600':['data/*.txt']},
    # any scripts will be copied into your $PATH, so that can run from the command line
    scripts = [],
    # some descriptions about this package (for searchability?)
    classifiers=[
      'Intended Audience :: Education',
      'Intended Audience :: Science/Research',
      'Programming Language :: Python',
      'Topic :: Scientific/Engineering :: Astronomy'
      ],
    # what other packages are required. these must be pip-installable
    install_requires=['numpy',
                      'scipy',
                      'ipython',
                      'matplotlib',
                      'jupyter',
                      'astropy'],

    # the packages in `key` will be installed if folks run `pip install henrietta[key]`
    extras_require={},
    # (I think just leave this set to False)
    zip_safe=False,
    # under what license is this code released?
    license='MIT')



import numpy as np
import matplotlib.pyplot as plt

def drawMoons(names, xpositions, xlim=[-500,500], labels=True):
    '''
    Draw a plot of the positions of moons relative to Jupiter.

    This function requires two input arguments. They should
    both be lists, and they should be the same size as each
    other. They are:

        moons -- a 1-dimensional list of moon names
        xpositions -- a 1-dimensional list of moon positions (in arcsec)

    For example:

        names = ['Io', 'Europa', 'Ganymede', 'Callisto']
        xpositions = [-20, 40, 80, -160]
        drawMoons(names, xpositions)

        (this should display a plot of the moon positions)

    Options keyword arguments

        xlim = [-500,500]
            This defines the x values of the left and
            right edges of the plotting range to be included.

        labels = True
            If the function is called with labels=True,
            then display the names of the moons.

            If the function is called with labels=False,
            then do not display the names of the moons.
    '''

    # since we're plotting only 1D positions, we make up y-values
    ypositions = np.zeros_like(xpositions)

    # we create a new figure, and set its size
    plt.figure(figsize=(10,0.5))

    # we plot the moons in their positions
    plt.plot(xpositions, ypositions,
                marker = '.',
                linewidth=0,
                color='black')

    # if desired, we add text labels to all the moons
    if labels:
        for x, y, n in zip(xpositions, ypositions, names):
            plt.text(x, y+0.5, n, ha='center', va='bottom', size=9)

    # plot Jupiter in the center
    plt.plot(0,0, marker='o', markersize=20, markerfacecolor='none', markeredgecolor='black')

    # set the x and y limits of the plot
    plt.xlim(*xlim)
    plt.ylim(-1,1)

    # turn off all axis labels (and the box around the plot)
    plt.axis('off')

    # make sure the plot shows to the screen
    plt.show()
